@@ -14,16 +14,18 @@ from inspect import getfullargspec, formatargspec, getcallargs
 
 from .encoders import BaseEncoder, UTF8Encoder
 from .exceptions import (
+        ConnectionLostError,
         Error,
         ErrorReply,
-        TransactionError,
-        NotConnectedError,
-        ConnectionLostError,
         NoRunningScriptError,
+        NotConnectedError,
         ScriptKilledError,
+        TimeoutError,
+        TransactionError,
 )
 from .replies import (
         BlockingPopReply,
+        BlockingPopPushReply,
         ClientListReply,
         ConfigPairReply,
         DictReply,
@@ -194,6 +196,7 @@ class PostProcessors:
                 str: cls.bytes_to_str,
                 bool: cls.int_to_bool,
                 BlockingPopReply: cls.multibulk_as_blocking_pop_reply,
+                BlockingPopPushReply: cls.bytes_to_blocking_poppush_reply,
                 ZRangeReply: cls.multibulk_as_zrangereply,
 
                 StatusReply: None,
@@ -266,9 +269,20 @@ class PostProcessors:
 
     @asyncio.coroutine
     def multibulk_as_blocking_pop_reply(protocol, result):
-        assert isinstance(result, MultiBulkReply)
-        list_name, value = yield from ListReply(result).aslist()
-        return BlockingPopReply(list_name, value)
+        if result is None:
+            raise TimeoutError('Timeout in blocking pop')
+        else:
+            assert isinstance(result, MultiBulkReply)
+            list_name, value = yield from ListReply(result).aslist()
+            return BlockingPopReply(list_name, value)
+
+    @asyncio.coroutine
+    def bytes_to_blocking_poppush_reply(protocol, result):
+        if result is None:
+            raise TimeoutError('Timeout in brpoplpush')
+        else:
+            assert isinstance(result, bytes)
+            return BlockingPopPushReply(protocol.decode_to_native(result))
 
     @asyncio.coroutine
     def multibulk_as_configpair(protocol, result):
@@ -472,6 +486,7 @@ class CommandCreator:
             try:
                 return {
                     BlockingPopReply: ":class:`BlockingPopReply <asyncio_redis.replies.BlockingPopReply>`",
+                    BlockingPopPushReply: ":class:`BlockingPopPushReply <asyncio_redis.replies.BlockingPopPushReply>`",
                     ConfigPairReply: ":class:`ConfigPairReply <asyncio_redis.replies.ConfigPairReply>`",
                     DictReply: ":class:`DictReply <asyncio_redis.replies.DictReply>`",
                     InfoReply: ":class:`InfoReply <asyncio_redis.replies.InfoReply>`",
@@ -504,7 +519,7 @@ class CommandCreator:
                     # XXX: Because of circulare references, we cannot use the real types here.
                     'Transaction': ":class:`asyncio_redis.Transaction`",
                     'Subscription': ":class:`asyncio_redis.Subscription`",
-                    'Script': ":returns: :class:`asyncio_redis.Script`",
+                    'Script': ":class:`~asyncio_redis.Script`",
                 }[type_]
             except KeyError:
                 if isinstance(type_, ListOf):
@@ -1396,19 +1411,23 @@ class RedisProtocol(asyncio.Protocol, metaclass=_RedisProtocolMeta):
 
     @_query_command
     def blpop(self, keys:ListOf(NativeType), timeout:int=0) -> BlockingPopReply:
-        """ Remove and get the first element in a list, or block until one is available. """
+        """ Remove and get the first element in a list, or block until one is available.
+        This will raise :class:`~asyncio_redis.exceptions.TimeoutError` when
+        the timeout was exceeded and Redis returns `None`. """
         return self._blocking_pop(b'blpop', keys, timeout=timeout)
 
     @_query_command
     def brpop(self, keys:ListOf(NativeType), timeout:int=0) -> BlockingPopReply:
-        """ Remove and get the last element in a list, or block until one is available. """
+        """ Remove and get the last element in a list, or block until one is available.
+        This will raise :class:`~asyncio_redis.exceptions.TimeoutError` when
+        the timeout was exceeded and Redis returns `None`. """
         return self._blocking_pop(b'brpop', keys, timeout=timeout)
 
     def _blocking_pop(self, command, keys, timeout:int=0):
         return self._query(command, *([ self.encode_from_native(k) for k in keys ] + [self._encode_int(timeout)]), set_blocking=True)
 
     @_query_command
-    def brpoplpush(self, source:NativeType, destination:NativeType, timeout:int=0) -> NativeType:
+    def brpoplpush(self, source:NativeType, destination:NativeType, timeout:int=0) -> BlockingPopPushReply:
         """ Pop a value from a list, push it to another list and return it; or block until one is available """
         return self._query(b'brpoplpush', self.encode_from_native(source), self.encode_from_native(destination),
                     self._encode_int(timeout), set_blocking=True)
@@ -1664,7 +1683,7 @@ class RedisProtocol(asyncio.Protocol, metaclass=_RedisProtocolMeta):
                 result = yield from subscription.next_published()
                 print(result)
 
-        :returns: :class:`asyncio_redis.Subscription`
+        :returns: :class:`~asyncio_redis.Subscription`
         """
         # (Make coroutine. @asyncio.coroutine breaks documentation. It uses
         # @functools.wraps to make a generator for this function. But _command
@@ -1893,7 +1912,11 @@ class RedisProtocol(asyncio.Protocol, metaclass=_RedisProtocolMeta):
 
     @_query_command
     def script_kill(self) -> StatusReply:
-        """ Kill the script currently in execution. """
+        """
+        Kill the script currently in execution.  This raises
+        :class:`~asyncio_redis.exceptions.NoRunningScriptError` when there are no
+        scrips running.
+        """
         try:
             return (yield from self._query(b'script', b'kill'))
         except ErrorReply as e:
@@ -1911,6 +1934,9 @@ class RedisProtocol(asyncio.Protocol, metaclass=_RedisProtocolMeta):
         Scripts are cached on the server side using the SCRIPT LOAD command.
 
         The return type/value depends on the script.
+
+        This will raise a :class:`~asyncio_redis.exceptions.ScriptKilledError`
+        exception if the script was killed.
         """
         if not keys: keys = []
         if not args: args = []
@@ -2144,6 +2170,9 @@ class Script:
 
             # If the LUA script returns something, retrieve the return value
             result = yield from script_reply.return_value()
+
+        This will raise a :class:`~asyncio_redis.exceptions.ScriptKilledError`
+        exception if the script was killed.
         """
         return self.get_evalsha_func()(self.sha, keys, args)
 
@@ -2188,6 +2217,9 @@ class Transaction:
     def exec(self):
         """
         Execute transaction.
+
+        This can raise a :class:`~asyncio_redis.exceptions.TransactionError`
+        when the transaction fails.
         """
         return self._protocol._exec()
 
